<?php


namespace Controller;

use Traits\TResponse;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Service\Call as CallService;

class Call
{
    use TResponse;

    private $callService;

    public function __construct(CallService $callService) {
        $this->callService = $callService;
    }

    public function findCalls(Request $request, Response $response) {
        $param = $request->getParameters();
       // var_dump(base64_decode($param['dynamic_filter']));
        ini_set('max_execution_time', '900');
        set_time_limit(900);
        try {
            $conditions = array();
            if(!empty($param['dynamic_filter'])){
                $conditions = \Util\Utilities::filterNextTOZeedhi($param['dynamic_filter']);
            }
            $cdProd = null;
            $clientes      = Utilities::getValueInFilterDataSourceByName($conditions,'CDCLIENTE', true);
            $cdProd        = Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCT_FILTER_ID', true);

            if(empty($clientes) && !is_array($clientes)){
                $clientes   = ["J26269316000177"];
            }
            if(empty($conditions)){
                $conditions = array (
                    array (
                        "columnName" => "STATUS",
                        "operator"	 => "IN",
                        "value"		 => ["O", "P", "W"]
                    ),
                    array (
                        "columnName" => "CALL_WITHOUT_LABELS_CHECK",
                        "operator"	 => "LIKE_I",
                        "value"		 => "%N%"
                    ),
                    array (
                        "columnName" => "REPORTED_BY",
                        "operator"	 => "IN",
                        "value"		 => ["000000002930"]
                    )
                );
            }
            $param = $request->getParameters();
            $order = array('UPDATED_AT.desc');
            if(!empty($param['order'])){
                $order = $param['order'];
            }
            $page  = null;
            if(!empty($param['page'])){
                $page = $param['page'];
            }
            $limit = null;
            if(!empty($param['limit'])){
                $limit = $param['limit'];
            }


            $data = $this->callService->findCalls($clientes, $cdProd, $conditions, $order, $page, $limit);
            $this->sendResponse($data, 200);
        } catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findCallComments(Request $request, Response $response) {
        try {
            $callType = $request->getParameter('CALL_TYPE');
            $callRelationship = $request->getParameter('CALL_RELATIONSHIP');
            $data = $this->callService->findCallComments($callRelationship, $callType);
            $this->sendResponse($data, 200);
        } catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findCallAttachments(Request $request, Response $response) {
        try {
            $callType = $request->getParameter('CALL_TYPE');
            $callRelationship = $request->getParameter('CALL_RELATIONSHIP');
            $param = $request->getParameters();
            $commentId = null;
            if(!empty($param['COMMENT_ID'])){
                $commentId = $param['COMMENT_ID'];
            }
            $filename  = null;
            if(!empty($param['FILENAME'])){
                $filename = $param['FILENAME'];
            }
            $data = $this->callService->findCallAttachments($callRelationship, $callType, $commentId, $filename);
            $this->sendResponse($data, 200);
        } catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCall(Request $request, Response $response)
    {
        try {
            $callType = $request->getParameter('CALL_TYPE');
            $callRelationship = $request->getParameter('CALL_RELATIONSHIP');
            $call = $this->callService->getCall($callRelationship, $callType);
            $this->sendResponse(array("data" => $call), 200);
        }catch (\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findProductCallProductsNew(Request $request, Response $response)
    {
        try {
            $param = $request->getParameters();
            $cdCliente = Utilities::getCdClienteBase();
            if(!empty($param['CDCLIENTE'])){
                $cdCliente = $param['CDCLIENTE'];
            }
            $callType = null;
            if(!empty($param['CALL_TYPE'])){
                $callType = $param['CALL_TYPE'];
            }
            $productCallProducts = $this->callService->findProductCallProductsNew($cdCliente,$callType);
            $this->sendResponse($productCallProducts, 200);

        } catch(\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }


    public function findProductCallClassification(Request $request, Response $response) {
        try {
            $productCallClassification = array();
            $applicationType =  $request->getParameter('APPLICATION_TYPE');

            if($applicationType === \Service\Call::TYPE_ISSUE) {
                $productCallClassification = $this->callService->findProductCallClassificationIssue();
            } else if($applicationType === \Service\Call::TYPE_RICH_CLIENT) {
                $productCallClassification = $this->callService->findProductCallClassificationFromRichClient();
            }
            $this->sendResponse($productCallClassification, 200);
        } catch(\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findProductCallProductTypes(Request $request, Response $response)
    {
        try {
            $productCode =  $request->getParameter('PRODUCT_CODE');

            $productCallProducTypes = $this->callService->findProductCallProductTypes($productCode);
            $this->sendResponse($productCallProducTypes, 200);
        } catch(\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findProductCallProductVersions(Request $request, Response $response)
    {
        try {
        $params = $request->getParameters();
        if(empty($params)) {
            $result = array(
                "pagination" => [
                    "total" => 2,
                    "page" => 1,
                    "limit" => 2
                ],
                "data" => []
            );
            $this->sendResponse($result, 200);
        } else {
            $product = $params["PRODUCT_TYPE"];
            $productCode = $params["PRODUCT_CONCAT"];

            $productCallProductVersions = $this->callService->findProductCalProductVersions($product, $product, $productCode);
            $this->sendResponse($productCallProductVersions);
        }
        } catch(\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findAllOperadorResponsible(Request $request, Response $response){
        try{
            $dataSet = $this->callService->findAllOperadorResponsible();
            $this->sendResponse($dataSet);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findAllCustomers(Request $request, Response $response){
        try{
            $dataSet = $this->callService->findAllCustomers();
            $this->sendResponse($dataSet);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findProductCallProductReleases(Request $request, Response $response){
        try{
            $params = $request->getParameters();
            $productType = $params["PRODUCT_TYPE"];
            $version = $params['PRODUCT_VERSION'];

            $productCallProductVersions = $this->callService->findProductCallProductReleases($productType, $version);
            $this->sendResponse($productCallProductVersions);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findProductCallProductPatch(Request $request, Response $response)
    {
        try {
            $params = $request->getParameters();

            $productType = $params["PRODUCT_TYPE"];
            $version = $params['PRODUCT_VERSION'];
            $release = $params['PRODUCT_RELEASE'];

            $productCallProductVersions = $this->callService->findProductCalProductPatch($productType, $version, $release);
            $this->sendResponse($productCallProductVersions);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findProductCallUseLicense(Request $request, Response $response)
    {
        try {

            $cdCliente =  $request->getParameter('CDCLIENTE');

            $productCallUserLicenses = $this->callService->findProductCallUseLicense($cdCliente);

            $this->sendResponse($productCallUserLicenses);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }
    public function findAllCallLabels(Request $request, Response $response) {
        try {

            $callLabels = $this->callService->findAllCallLabels();
            $this->sendResponse($callLabels);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findMenuItems(Request $request, Response $response)
    {
        try{
            $productId =  $request->getParameter('PRODUCT_ID');
            $cdCliente =  $request->getParameter('CDCLIENTE');
            $menus = $this->callService->findMenuItems($productId,$cdCliente);
            $this->sendResponse($menus);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findProductVersion(Request $request, Response $response)
    {
        try{
            $productId =  $request->getParameter('PRODUCT_ID');
            $menus = $this->callService->findProductVersion($productId);
            $this->sendResponse($menus);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function productCallSave(Request $request, Response $response)
    {
        ini_set('max_execution_time', '60');
        set_time_limit(60);
        try {
            $productCall = $request->getParameters();
            $id             = empty($productCall['ID']) ? null : $productCall['ID'];
            $callType       = empty($productCall['TYPE']) ? null : $productCall['TYPE'];
            $reference      = empty($productCall['REFERENCE']) ? null : $productCall['REFERENCE'];
            $product        = empty($productCall['PRODUCT']) ? null : $productCall['PRODUCT'];
            $projectId      = empty($productCall['PROJECT_ID']) ? null : $productCall['PROJECT_ID'];
            $board          = empty($productCall['BOARD']) ? null : $productCall['BOARD'];
            $classification = empty($productCall['CLASSIFICATION']) ? null : $productCall['CLASSIFICATION'];
            $description    = empty($productCall['DESCRIPTION']) ? null : $productCall['DESCRIPTION'];
            $callLabels     = empty($productCall['CALL_LABELS']) ? array() : $productCall['CALL_LABELS'];
            $productType    = empty($productCall['PRODUCT_TYPE']) ? null : $productCall['PRODUCT_TYPE'];
            $productVersion = isset($productCall['PRODUCT_VERSION']) ? $productCall['PRODUCT_VERSION'] : null;
            $productRelease = isset($productCall['PRODUCT_RELEASE']) ? $productCall['PRODUCT_RELEASE'] : null;
            $productPatch   = isset($productCall['PRODUCT_PATCH']) ? $productCall['PRODUCT_PATCH'] : null;
            $useLicense     = empty($productCall['USE_LICENSE']) ? null : $productCall['USE_LICENSE'];
            $reportedBy     = empty($productCall['REPORTED_BY']) ?  null : $productCall['REPORTED_BY'];
            $cdcliente      = empty($productCall['CDCLIENTE']) ?  null : $productCall['CDCLIENTE'];
            $appointmentDate     = empty($productCall['APPOINTMENT_DATE']) ?  null : \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE,$productCall['APPOINTMENT_DATE']);
            $devopsId = empty($productCall['DEVOPS_ID']) ? null : $productCall['DEVOPS_ID'];
            $menuItemId = empty($productCall['MENU_ITEM_ID']) ? null : $productCall['MENU_ITEM_ID'];
            $menuVersionId = empty($productCall['MENU_VERSION_ID']) ? null : $productCall['MENU_VERSION_ID'];
            if(empty($product)){
                $product = $projectId;
            }
            if(empty($reportedBy)){
                $reportedBy = empty($productCall['REPORTER_BY']) ?  null : $productCall['REPORTER_BY'];
            }
            $callRelationship = empty($productCall['CALL_RELATIONSHIP']) ? null : $productCall['CALL_RELATIONSHIP'];
            if(empty($classification)){
                $classification = empty($productCall['CDMOTICONT']) ? null : $productCall['CDMOTICONT'];
            }
            if(!$callRelationship){
                $newProductCall = $this->callService->saveNewProductCall($callType, $reference, $product, $productType,
                    $productVersion, $useLicense, $classification, $board, $description, $reportedBy,
                    $callLabels, $productRelease, $productPatch, $callRelationship,$devopsId,$menuItemId,$menuVersionId,$cdcliente,null,$appointmentDate);
                $productCall['ID'] = $newProductCall['ID'];
                $productCall['CALL_RELATIONSHIP'] = $newProductCall['CALL_RELATIONSHIP'];
            } else {
                $this->callService->updateProductCall($callType, $id,  $reference, $product, $productType, $productVersion,
                    $useLicense, $classification, $board, $description, $reportedBy, $callLabels, $devopsId, $menuItemId,
                    $menuVersionId, $cdcliente,$callRelationship,$appointmentDate,$productRelease,$productPatch);
            }
            $result = array(
                "pagination" => [
                    "total" => 2,
                    "page" => 1,
                    "limit" => 2
                ],
                "data" => $productCall
            );
            $this->sendResponse($result);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function questionForSendCall(Request $request, Response $response)
    {
        try {
            $param = $request->getParameters();

            $lang = empty($param['LANG']) ? 'pt' : $param['LANG'];
            $callRelationship = empty($param['CALL_RELATIONSHIP']) ? null : $param['CALL_RELATIONSHIP'];
            $callType = empty($param['CALL_TYPE']) ? null : $param['CALL_TYPE'];

            $productCalls = $this->callService->getQuestionForSendCall($lang, $callType, $callRelationship);
            $this->sendResponse($productCalls);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function operadorByResponse(Request $request, Response $response)
    {
        try {
            $param = $request->getParameters();
            $callRelationship = empty($param['CALL_RELATIONSHIP']) ? null : $param['CALL_RELATIONSHIP'];
            $callType = empty($param['CALL_TYPE']) ? null : $param['CALL_TYPE'];
            $cdfastramdes = empty($param['CDFASTRAMDES']) ? null : $param['CDFASTRAMDES'];
            $nrfastramdes = empty($param['NRFASTRAMDES']) ? null : $param['NRFASTRAMDES'];
            $cdcliente = empty($param['CDCLIENTE']) ? null : $param['CDCLIENTE'];

            $productCalls = $this->callService->operadorByResponse($callType, $cdfastramdes, $nrfastramdes, $callRelationship,$cdcliente);
            $this->sendResponse($productCalls);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function sendCallGep(Request $request, Response $response)
    {
        try {
            $row                =  $request->getParameters();
            $cdfastramdes       = $row['CDFASTRAMDES'];
            $cdoperador         = $row['CDOPERADOR'];
            $comment            = $row['COMMENT'];
            $nrfastramdes       = $row['NRFASTRAMDES'];
            $txacaofatram       = $row['TXACAOFATRAM'];
            $callRelationship   = $row['CALL_RELATIONSHIP'];
            $document           = empty($row['DOCUMENT']) ?  array() : $row['DOCUMENT'];
            $callType           = $row['CALL_TYPE'];
            $this->callService->sendCallGep(null, $cdfastramdes, $cdoperador, $comment, $nrfastramdes, $txacaofatram,
                $callType, $callRelationship, $document);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findCurrentUser(Request $request, Response $response)
    {
        try {
            $this->sendResponse($this->callService->findCurrentUser());
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

}
