import { Metadata, Http } from '@zeedhi/core';
import { ITabsBeforeChangeEventParam, Modal, ModalService, LoadingService } from '@zeedhi/common';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';
import { InstanceService } from '@/services/InstanceService';
import Messages from '@/services/Messages';
import { IterableComponentRender, TekRestDatasource } from '@zeedhi/teknisa-components-common';
import { FileUpload } from '@/services/FileUpload';

export class DemandManagementController {
	public defaultTimeoutTimer: any;

	public baseUrl = '';

	public callType = '';

	public callRelationship = '';

	public callID = '';

	public isLoadingDefault = false;

	public showDemandManagementForm = false;

    get isVisibleDemandManagementGrid() {
		return !this.showDemandManagementForm;
    }

	private modalAddCall!: Modal;

	private modalForwardCall!: Modal;

	private modalFilterCall!: Modal;

	private modalAddComments!: Modal;

	private modalUploadFile!: Modal;

	// START SHARED VARIABLES ADD/EDIT ---
	private typeIsI = false;

	private typeIsS = false;

	public clientNOTSelected = true;

	public get disableProduct() {
		if (this.clientNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public productNOTSelected = true;

	public get disableCetegory() {
		if (this.productNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public screenNameNOTSelected = true;

	public get disableVersionI() {
		if (this.screenNameNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public productTypeNOTSelected = true;

	public get disableVersionS() {
		if (this.productTypeNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public versionNOTSelected = true;

	public get disableRelease() {
		if (this.versionNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public releaseNOTSelected = true;

	public get disablePatch() {
		if (this.releaseNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}
	// END SHARED VARIABLES ADD/EDIT ---

	public async onMountedGeneralCard() {
		this.baseUrl = process.env.VUE_APP_END_POINT;
	}

	public async onMountedDemandManagementGrid() {
		this.modalFilterCall = await ModalService.createFromJson('demandManagement/modal/filterCallDemandManagementModal', true);
		this.modalAddCall = await ModalService.createFromJson('demandManagement/modal/addCallDemandManagementModal', true);
		this.modalForwardCall = await ModalService.createFromJson('demandManagement/modal/forwardCallDemandManagementModal', true);
		const currentUser = await Http.get('findCurrentUser');
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		if (callGridInstance.datasource instanceof TekRestDatasource) {
			console.log('entrou');
			callGridInstance.datasource.dynamicFilter = {
				REPORTED_BY: [
					{
						operation: 'IN',
						relation: 'AND',
						value: [currentUser.data.CDOPERADOR],
					},
				],
				STATUS: [
					{
						operation: 'IN',
						relation: 'AND',
						value: ['O', 'P', 'W'],
					},
				],
			};
		}
		try {
			LoadingService.show('Carregando');
			this.hideFilterCallModal();
			await callGridInstance.datasource.get();
			LoadingService.hide();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
			LoadingService.hide();
		}
	}

	public onDestroyDemandManagementGrid() {
		this.modalFilterCall?.destroy();
		this.modalAddCall?.destroy();
		this.modalForwardCall?.destroy();
	}

	// START CALL GRID ---
	public publicComment = false;

	public checkForwardCall(params: any) {
		return params.FORWARD_CALL_GRID === 'S';
		// console.log(params);
		// return true;
	}

	public async rowClickGridDemandManagement(params: any) {
		LoadingService.show('Carregando');
		this.publicComment = false;
		this.showDemandManagementForm = true;
		this.callRelationship = params.row.CALL_RELATIONSHIP;
		this.callType = params.row.TYPE;
		this.callID = params.row.ID;
		if (params.row.TYPE === 'I') {
			this.typeIsI = true;
			this.typeIsS = false;
			await this.getSelectsEditI();
			this.setIValidationsEDIT();
			this.removeSValidationsEDIT();
		} else {
			this.typeIsI = false;
			this.typeIsS = true;
			await this.getSelectsEditS();
			this.setSValidationsEDIT();
			this.removeIValidationsEDIT();
		}

		// await this.fillCallForm();
		LoadingService.hide();
		// const callFormInstance = Metadata.getInstance<Form>('formDemandManagement');
		// console.log(callFormInstance);
	}

	public async getSelectsEditI() {
		// const reportedByInstance = InstanceService.getInstanceZdSelect('reportedByEdit');
		// const currentCustomerInstance = InstanceService.getInstanceZdSelect('customerEdit');
		const productInstance = InstanceService.getInstanceZdSelect('productEdit');
		const categoryInstance = InstanceService.getInstanceZdSelect('categoryEdit');
		const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEdit');
		const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
		try {
			const callData = await Http.get('call', { params: { CALL_RELATIONSHIP: this.callRelationship, CALL_TYPE: this.callType } });
			console.log(callData);
			// Aplicar Filtros nos Selects
			productInstance.datasource.filter.CDCLIENTE = callData.data.data.CDCLIENTE;
			productInstance.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
			productInstance.datasource.filter.CALL_TYPE = this.callType;
			await productInstance.datasource.get();
			categoryInstance.datasource.filter.APPLICATION_TYPE = 'I';
			await categoryInstance.datasource.get();
			screenNameInstance.datasource.filter.CDCLIENTE = callData.data.data.CDCLIENTE;
			screenNameInstance.datasource.filter.PRODUCT_ID = callData.data.data.PRODUCT;
			await screenNameInstance.datasource.get();
			versionIInstance.datasource.filter.PRODUCT_ID = callData.data.data.PRODUCT;
			await versionIInstance.datasource.get();
			// Atribuir Valor aos selects
			const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
			callFormInstance.value.customerIdEdit = callData.data.data.ID; // I S
			callFormInstance.value.teknisaIdEdit = callData.data.data.CALL_RELATIONSHIP; // I S
			callFormInstance.value.reportedByEdit = callData.data.data.REPORTED_BY; // I S
			callFormInstance.value.referenceEdit = callData.data.data.REFERENCE; // I S
			callFormInstance.value.customerEdit = callData.data.data.CDCLIENTE; // I S
			this.clientNOTSelected = false;
			// productInstance.value = callData.data.data.PRODUCT; // I S
			// this.productNOTSelected = false;
			// callFormInstance.value.categoryEdit = callData.data.data.CLASSIFICATION; // I S
			// screenNameInstance.value = callData.data.data.MENU_ITEM_ID; // I
			// this.screenNameNOTSelected = false;
			// versionIInstance.value = callData.data.data.MENU_VERSION; // I
			// this.versionNOTSelected = false;
			callFormInstance.value.devVersionEdit = callData.data.data.VERSION_DEV; // I
			callFormInstance.value.initialAppointmentDateEdit = callData.data.data.APPOINTMENT_DATE_INIT; // I S
			callFormInstance.value.appointmentDateEdit = callData.data.data.APPOINTMENT_DATE; // I S
			callFormInstance.value.labelsEdit = callData.data.data.CALL_LABELS; // I S
			callFormInstance.value.descriptionEdit = callData.data.data.DESCRIPTION; // I S
			// await reportedByInstance.datasource.get();
			// await currentCustomerInstance.datasource.get();
			// await currentProductInstance.datasource.get();
			// if (callData.data.data.PRODUCT) {
			// 	const currentScreenName = InstanceService.getInstanceZdSelect('screenNameEdit');
			// 	currentScreenName.datasource.filter.
			// }
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}

	public async getSelectsEditS() {
		// const reportedByInstance = InstanceService.getInstanceZdSelect('reportedByEdit');
		// const currentCustomerInstance = InstanceService.getInstanceZdSelect('customerEdit');
		const productInstance = InstanceService.getInstanceZdSelect('productEdit');
		const categoryInstance = InstanceService.getInstanceZdSelect('categoryEdit');
		// const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
		// const releaseInstance = InstanceService.getInstanceZdSelect('releaseAdd');
		// const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
		try {
			const callData = await Http.get('call', { params: { CALL_RELATIONSHIP: this.callRelationship, CALL_TYPE: this.callType } });
			console.log(callData);
			// Aplicar Filtros nos Selects
			productInstance.datasource.filter.CDCLIENTE = callData.data.data.CDCLIENTE;
			productInstance.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
			productInstance.datasource.filter.CALL_TYPE = this.callType;
			await productInstance.datasource.get();
			categoryInstance.datasource.filter.APPLICATION_TYPE = 'S';
			await categoryInstance.datasource.get();
			// Atribuir Valor aos selects
			const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
			callFormInstance.value.customerIdEdit = callData.data.data.ID; // I S
			callFormInstance.value.teknisaIdEdit = callData.data.data.CALL_RELATIONSHIP; // I S
			callFormInstance.value.reportedByEdit = callData.data.data.REPORTED_BY; // I S
			callFormInstance.value.referenceEdit = callData.data.data.REFERENCE; // I S
			callFormInstance.value.customerEdit = callData.data.data.CDCLIENTE; // I S
			this.clientNOTSelected = false;
			// productInstance.value = callData.data.data.PRODUCT; // I S
			// this.productNOTSelected = false;
			// callFormInstance.value.categoryEdit = callData.data.data.CLASSIFICATION; // I S
			// this.productTypeNOTSelected = false;
			// this.releaseNOTSelected = false;
			callFormInstance.value.initialAppointmentDateEdit = callData.data.data.APPOINTMENT_DATE_INIT; // I S
			callFormInstance.value.appointmentDateEdit = callData.data.data.APPOINTMENT_DATE; // I S
			callFormInstance.value.labelsEdit = callData.data.data.CALL_LABELS; // I S
			callFormInstance.value.descriptionEdit = callData.data.data.DESCRIPTION; // I S
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}
	// END CALL GRID -----

	// START MODAL FILTER CALL ----
	public showFilterCallModal(params: any) {
		params.event.preventDefault();
		this.modalFilterCall.show();
	}

	public async saveFilterCallModal() {
		const filterFormInstance = InstanceService.getInstanceZdForm('filterFormFilterCall');
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		if (callGridInstance.datasource instanceof TekRestDatasource) {
			callGridInstance.datasource.dynamicFilter = {
				NRCALL: [// OK
					{
						operation: '=',
						relation: 'AND',
						value: filterFormInstance.value.idClienteFilterCall,
					},
				],
				CALL_RELATIONSHIP: [// OK
					{
						operation: 'LIKE_I',
						relation: 'AND',
						value: filterFormInstance.value.idTeknisaFilterCall ? `%${filterFormInstance.value.idTeknisaFilterCall.split(' ').join('%')}%` : filterFormInstance.value.idTeknisaFilterCall,
					},
				],
				REFERENCE: [// OK
					{
						operation: 'LIKE_I',
						relation: 'AND',
						value: filterFormInstance.value.referenceFilterCall ? `%${filterFormInstance.value.referenceFilterCall.split(' ').join('%')}%` : filterFormInstance.value.referenceFilterCall,
					},
				],
				STATUS: [
					{
						operation: 'IN',
						relation: 'AND',
						value: filterFormInstance.value.situationFilterCall,
					},
				],
				REPORTED_BY: [// OK
					{
						operation: 'IN',
						relation: 'AND',
						value: filterFormInstance.value.reportedByFilterCall,
					},
				],
				CDOPERRESP: [
					{
						operation: 'IN',
						relation: 'AND',
						value: filterFormInstance.value.responsibleFilterCall,
					},
				],
				PRODUCT_FILTER_ID: [
					{
						operation: 'IN',
						relation: 'AND',
						value: filterFormInstance.value.productFilterCall,
					},
				],
				CLASSIFICATION: [
					{
						operation: 'IN',
						relation: 'AND',
						value: filterFormInstance.value.categoryFilterCall,
					},
				],
				CREATED_AT: [
					{
						operation: 'BETWEEN',
						relation: 'AND',
						value: filterFormInstance.value.createdAtFilterCall,
					},
				],
				UPDATED_AT: [
					{
						operation: 'BETWEEN',
						relation: 'AND',
						value: filterFormInstance.value.updatedAtFilterCall,
					},
				],
				CDCLIENTE: [
					{
						operation: 'IN',
						relation: 'AND',
						value: filterFormInstance.value.customerFilterCall,
					},
				],
				NRCALLLABEL: [
					{
						operation: 'IN',
						relation: 'AND',
						value: filterFormInstance.value.labelsFilterCall,
					},
				],
				CALL_WITHOUT_LABELS_CHECK: [
					{
						operation: 'LIKE_I',
						relation: 'AND',
						value: filterFormInstance.value.noLabelFilterCall,
					},
				],
			};
		}
		try {
			LoadingService.show('Carregando');
			this.hideFilterCallModal();
			await callGridInstance.datasource.get();
			LoadingService.hide();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
			LoadingService.hide();
		}
	}

	public hideFilterCallModal() {
		this.modalFilterCall.hide();
	}
	// END MODAL FILTER CALL ----

	// START MODAL ADD CALL ----
	public addButtonDemandManagementGrid() {
		this.modalAddCall.show();
	}

	public onInputClientAddCallModal(params: any) { // CLIENT INPUT ADD
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			const productInstance = InstanceService.getInstanceZdSelect('productAdd');
			productInstance.value = '';
			this.typeIsI = false;
			this.typeIsS = false;
			this.resetValidations();
			if (params.value) {
				this.resetInputtedValuesADD();
				productInstance.value = '';
				productInstance.datasource.filter.CDCLIENTE = params.component.value;
				try {
					await productInstance.datasource.get();
					this.clientNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.clientNOTSelected = true;
					this.productNOTSelected = true;
					this.screenNameNOTSelected = true;
					this.productTypeNOTSelected = true;
					this.versionNOTSelected = true;
					this.releaseNOTSelected = true;
					this.removeIValidations();
					this.removeSValidations();
				}
			} else {
				this.clientNOTSelected = true;
				this.productNOTSelected = true;
				this.screenNameNOTSelected = true;
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				this.resetInputtedValuesADD();
				this.removeIValidations();
				this.removeSValidations();
			}
		}, 350);
	}

	public onInputProductAddCallModal(params: any) { // PRODUCT ADD
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			if (params.value) {
				this.resetInputtedValuesADD();
				const customerInstance = InstanceService.getInstanceZdSelect('customerAdd');
				const categoryInstance = InstanceService.getInstanceZdSelect('categoryAdd');
				if (params.component.selectValue.PLATAFORMA === 'MOBILE') {
					const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameAdd');
					this.setIValidations();
					this.removeSValidations();
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'I';
					screenNameInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					screenNameInstance.datasource.filter.PRODUCT_ID = params.component.value;
					try {
						await categoryInstance.datasource.get();
						await screenNameInstance.datasource.get();
						this.typeIsI = true;
						this.typeIsS = false;
						this.productNOTSelected = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.typeIsI = false;
						this.typeIsS = false;
						this.productNOTSelected = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
						this.removeIValidations();
						this.removeSValidations();
					}
				} else {
					const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
					const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseAdd');
					this.setSValidations();
					this.removeIValidations();
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'S';
					productTypeInstance.datasource.filter.PRODUCT_CODE = params.component.value;
					useLicenseInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					try {
						await categoryInstance.datasource.get();
						await productTypeInstance.datasource.get();
						await useLicenseInstance.datasource.get();
						this.typeIsI = false;
						this.typeIsS = true;
						this.productNOTSelected = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.typeIsI = false;
						this.typeIsS = false;
						this.productNOTSelected = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
						this.removeIValidations();
						this.removeSValidations();
					}
				}
			} else {
				this.typeIsI = false;
				this.typeIsS = false;
				this.productNOTSelected = true;
				this.screenNameNOTSelected = true;
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				this.removeIValidations();
				this.removeSValidations();
				this.resetInputtedValuesADD();
			}
		}, 350);
	}

	public resetInputtedValuesADD() { // I S
		const addCallFormInstance = InstanceService.getInstanceZdForm('formAddCall');
		addCallFormInstance.value.screenNameAdd = '';
		addCallFormInstance.value.versionIAdd = '';
		addCallFormInstance.value.productTypeAdd = '';
		addCallFormInstance.value.versionSAdd = '';
		addCallFormInstance.value.releaseAdd = '';
		addCallFormInstance.value.patchAdd = '';
		addCallFormInstance.value.useLicenseAdd = '';
	}

	public setIValidations() {
		const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameAdd');
		const versionIInstance = InstanceService.getInstanceZdSelect('versionIAdd');
		screenNameInstance.addValidation('required');
		versionIInstance.addValidation('required');
	}

	public removeIValidations() {
		const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameAdd');
		const versionIInstance = InstanceService.getInstanceZdSelect('versionIAdd');
		screenNameInstance.removeValidation('required');
		versionIInstance.removeValidation('required');
	}

	public setSValidations() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseAdd');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseAdd');
		const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
		productTypeInstance.addValidation('required');
		useLicenseInstance.addValidation('required');
		versionSInstance.addValidation('required');
		releaseInstance.addValidation('required');
		patchInstance.addValidation('required');
	}

	public removeSValidations() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseAdd');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseAdd');
		const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
		productTypeInstance.removeValidation('required');
		useLicenseInstance.removeValidation('required');
		versionSInstance.removeValidation('required');
		releaseInstance.removeValidation('required');
		patchInstance.removeValidation('required');
	}

	public onInputScreenNameAddCallModal(params: any) { // SCREEN NAME
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			const versionIInstance = InstanceService.getInstanceZdSelect('versionIAdd');
			if (params.value) {
				versionIInstance.datasource.filter.PRODUCT_ID = params.component.value;
				try {
					await versionIInstance.datasource.get();
					this.screenNameNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.screenNameNOTSelected = true;
				}
			} else {
				this.screenNameNOTSelected = true;
				versionIInstance.value = '';
			}
		}, 350);
	}

	public onInputProductTypeAddCallModal(params: any) { // PRODUCT TYPE
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseAdd');
			const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
			if (params.value) {
				const productInstance = InstanceService.getInstanceZdSelect('productAdd');
				versionSInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
				versionSInstance.datasource.filter.PRODUCT_CONCAT = `${params.component.selectedData.CDARVPROD}${productInstance.selectedData.SGSISTEMA}`;
				try {
					const version = await versionSInstance.datasource.get();
					if (version.length === 1) {
						this.versionNOTSelected = false;
						versionSInstance.value = version[0].ID;
						releaseInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
						releaseInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
						const release = await releaseInstance.datasource.get();
						if (release.length === 1) {
							this.releaseNOTSelected = false;
							releaseInstance.value = release[0].ID;
							patchInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
							patchInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
							patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
							await patchInstance.datasource.get();
						}
					}
					this.productTypeNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.productTypeNOTSelected = true;
					this.versionNOTSelected = true;
					this.releaseNOTSelected = true;
					versionSInstance.value = '';
					releaseInstance.value = '';
					patchInstance.value = '';
				}
			} else {
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				versionSInstance.value = '';
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		}, 350);
	}

	public onInputVersionSAddCallModal(params: any) { // VERSION S (DELPHI)
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseAdd');
			const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
				releaseInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				releaseInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
				try {
					const release = await releaseInstance.datasource.get();
					if (release.length === 1) {
						releaseInstance.value = release[0].ID;
						this.releaseNOTSelected = false;
						patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
						patchInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
						patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
						await patchInstance.datasource.get();
					}
					this.versionNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.versionNOTSelected = true;
					this.releaseNOTSelected = true;
					releaseInstance.value = '';
					patchInstance.value = '';
				}
			} else {
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		}, 350);
	}

	public onInputReleaseAddCallModal(params: any) { // RELEASE
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
				patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				patchInstance.datasource.filter.PRODUCT_VERSION = versionSInstance.selectedData.ID;
				patchInstance.datasource.filter.PRODUCT_RELEASE = params.component.value;
				try {
					await patchInstance.datasource.get();
					this.releaseNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.releaseNOTSelected = true;
					patchInstance.value = '';
				}
			} else {
				this.releaseNOTSelected = true;
				patchInstance.value = '';
			}
		}, 350);
	}

	public async saveAddNewCall() { // SAVE
		const addCallFormInstance = InstanceService.getInstanceZdForm('formAddCall');
		const checkValidation = addCallFormInstance.validate();
		if (checkValidation) {
			this.isLoadingDefault = true;
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const customerInstance = InstanceService.getInstanceZdSelect('customerAdd');
			if (this.typeIsI) {
				const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameAdd');
				try {
					await Http.post('/productCallSave', {
						REFERENCE: addCallFormInstance.value.referenceAdd,
						CALL_LABELS: addCallFormInstance.value.labelsAdd,
						DESCRIPTION: addCallFormInstance.value.descriptionAdd,
						REPORTED_BY: addCallFormInstance.value.reportedByAdd,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'MOBILE',
						PROJECT_ID: addCallFormInstance.value.productAdd,
						SGSISTEMA: null,
						TYPE: 'I',
						PRODUCT_VERSION: '0',
						PRODUCT_RELEASE: '0',
						MENU_ITEM_ID: screenNameInstance.selectedData.ID,
						DEVOPS_ID: screenNameInstance.selectedData.NREXTERNAL_PRODUCT_ID,
						MENU_VERSION_ID: addCallFormInstance.value.versionIAdd,
						CLASSIFICATION: addCallFormInstance.value.categoryAdd,
						APPOINTMENT_DATE: addCallFormInstance.value.appointmentDateAdd,
					});
					AlertServiceCustom.defaultSaveSuccessAlert();
					this.closeAddModalDemandManagementGrid();
					await callGridInstance.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			} else {
				const productInstance = InstanceService.getInstanceZdSelect('productAdd');
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
				try {
					await Http.post('/productCallSave', {
						REFERENCE: addCallFormInstance.value.referenceAdd,
						PRODUCT_TYPE_NAME: productTypeInstance.selectedData.LABEL,
						MENU_ITEM: '',
						MENU_VERSION: '',
						NMRAZSOCCLIE: customerInstance.selectedData.NMRAZSOCCLIE,
						CALL_LABELS: addCallFormInstance.value.labelsAdd,
						DESCRIPTION: addCallFormInstance.value.descriptionAdd,
						REPORTED_BY: addCallFormInstance.value.reportedByAdd,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'DELPHI',
						PROJECT_ID: addCallFormInstance.value.productAdd,
						SGSISTEMA: productInstance.selectedData.SGSISTEMA,
						TYPE: 'S',
						PRODUCT: versionSInstance.selectedData.CDPRODCOMP,
						PRODUCT_VERSION: addCallFormInstance.value.versionSAdd,
						PRODUCT_RELEASE: addCallFormInstance.value.releaseAdd,
						PRODUCT_TYPE: addCallFormInstance.value.productTypeAdd,
						CDARVPROD: productTypeInstance.selectedData.CDARVPROD,
						CLASSIFICATION: addCallFormInstance.value.categoryAdd,
						PRODUCT_PATCH: addCallFormInstance.value.patchAdd,
						USE_LICENSE: addCallFormInstance.value.useLicenseAdd,
						APPOINTMENT_DATE: addCallFormInstance.value.appointmentDateAdd,
					});
					AlertServiceCustom.defaultSaveSuccessAlert();
					this.closeAddModalDemandManagementGrid();
					await callGridInstance.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			}
			this.isLoadingDefault = false;
		}
	}

	public resetValidations() {
		const addCallInstance = InstanceService.getInstanceZdForm('formAddCall');
		addCallInstance.resetValidation();
	}

	public closeAddModalDemandManagementGrid() {
		this.modalAddCall.hide();
		this.typeIsI = false;
		this.typeIsS = false;
		this.clientNOTSelected = true;
		this.productNOTSelected = true;
		this.screenNameNOTSelected = true;
		this.productTypeNOTSelected = true;
		this.versionNOTSelected = true;
		this.releaseNOTSelected = true;
	}
	// END MODAL ADD CALL ----

	// START MODAL FORWARD CALL ---
	public currentCustomerForwardCall = '';

	public questionLabelForwardCall = '';

	public disableForwardTo = true;

	public lastCommentLabel = '';

	public forwardCallType = '';

	public forwardCallRelationship = '';

	public forwardCDFASTRAMDES = '';

	public forwardNRFASTRAMDES = '';

	public forwardTXACAOFATRAM = '';

	public get forwardToDisable() {
		if (this.disableForwardTo) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public async clickOpenForwardCall(params: any) {
		params.event.stopPropagation();
		await this.modalForwardCall.show();
		this.currentCustomerForwardCall = params.row.CDCLIENTE;
		this.forwardCallType = params.row.TYPE;
		this.forwardCallRelationship = params.row.CALL_RELATIONSHIP;
		const questionSelectInstance = InstanceService.getInstanceZdSelect('questionSelectForwardCallModal');
		questionSelectInstance.datasource.filter.CALL_TYPE = this.forwardCallType;
		questionSelectInstance.datasource.filter.CALL_RELATIONSHIP = this.forwardCallRelationship;
		questionSelectInstance.datasource.filter.LANG = 'pt';
		try {
			const questionsTest = await questionSelectInstance.datasource.get();
			this.questionLabelForwardCall = `${questionsTest[0].TXPERGFATRAM}*`;
			const sendObject = { params: {
					CALL_TYPE: this.forwardCallType,
					CALL_RELATIONSHIP: this.forwardCallRelationship,
				},
			};
			const lastComment = await Http.get('/callComments', sendObject);
			if (lastComment.data.data.length > 0) {
				this.lastCommentLabel = lastComment.data.data[0].DESCRIPTION;
			} else {
				this.lastCommentLabel = 'Não possui cometário!';
			}
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
			this.closeModalForwardCall();
		}
	}

	public onInputQuestionForwardCall(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			const forwardToInstance = InstanceService.getInstanceZdSelect('forwardToForwardCallModal');
			if (params.value) {
				this.resetValidationsForwardCall();
				forwardToInstance.datasource.filter.CDCLIENTE = this.currentCustomerForwardCall;
				forwardToInstance.datasource.filter.CALL_TYPE = this.forwardCallType;
				forwardToInstance.datasource.filter.CALL_RELATIONSHIP = this.forwardCallRelationship;
				forwardToInstance.datasource.filter.CDFASTRAMDES = params.component.selectedData.CDFASTRAMDES;
				forwardToInstance.datasource.filter.NRFASTRAMDES = params.component.selectedData.NRFASTRAMDES;
				try {
					const whoFowardResponse = await forwardToInstance.datasource.get();
					if (whoFowardResponse.length === 1) {
						forwardToInstance.value = whoFowardResponse[0].CDOPERADOR;
						this.disableForwardTo = true;
					} else {
						this.disableForwardTo = false;
					}
					this.forwardTXACAOFATRAM = params.component.selectedData.TXACAOFATRAM;
					this.forwardCDFASTRAMDES = params.component.selectedData.CDFASTRAMDES;
					this.forwardNRFASTRAMDES = params.component.selectedData.NRFASTRAMDES;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					forwardToInstance.value = '';
					this.disableForwardTo = true;
					this.forwardCDFASTRAMDES = '';
					this.forwardNRFASTRAMDES = '';
					this.forwardTXACAOFATRAM = '';
				}
			} else {
				forwardToInstance.value = '';
				this.disableForwardTo = true;
				this.forwardCDFASTRAMDES = '';
				this.forwardNRFASTRAMDES = '';
				this.forwardTXACAOFATRAM = '';
			}
		}, 350);
	}

	public async saveForwardCall() {
		const forwardCallInstance = InstanceService.getInstanceZdForm('formForwardCallModal');
		const checkValidation = forwardCallInstance.validate();
		if (checkValidation) {
			this.isLoadingDefault = true;
			const fileInput: any = Metadata.getInstance('fileInputForwardCallModal');
			if (fileInput.value && fileInput.value.length > 0) {
				let mimeCheck = true;
				const checkFile = fileInput.value.map((file: any) => {
					if ((file.type === 'application/msword'
					|| file.type === 'image/jpeg'
					|| file.type === 'image/png'
					|| file.type === 'application/pdf'
					|| file.type === 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
					|| file.type === 'application/vnd.ms-excel.sheet.macroenabled.12'
					|| file.type === 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
					|| file.type === 'application/vnd.ms-excel'
					|| file.type === 'text/plain'
					|| file.type === 'application/zip')
					&& file.size < 50000000) {
						return FileUpload.buildFileSendObject(file);
					}
					mimeCheck = false;
					return false;
				});
				if (mimeCheck) {
					Promise.all(checkFile).then(async (fileArray) => {
						try {
							await Http.post('/sendCallGep', {
								CALL_RELATIONSHIP: this.forwardCallRelationship,
								CALL_TYPE: this.forwardCallType,
								CDFASTRAMDES: this.forwardCDFASTRAMDES,
								CDOPERADOR: forwardCallInstance.value.forwardToForwardCallModal,
								COMMENT: forwardCallInstance.value.observationForwardCallModal,
								DOCUMENT: fileArray,
								NRFASTRAMDES: this.forwardNRFASTRAMDES,
								TXACAOFATRAM: this.forwardTXACAOFATRAM,
								requestType: 'DataSet',
							});
							this.closeModalForwardCall();
							AlertServiceCustom.defaultSaveSuccessAlert();
							this.isLoadingDefault = false;
						} catch (e) {
							DialogServiceCustom.showErrorBackend(e);
							this.isLoadingDefault = false;
						}
					});
				} else {
					AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_TYPE);
					this.isLoadingDefault = false;
				}
			} else {
				try {
					await Http.post('/sendCallGep', {
						CALL_RELATIONSHIP: this.forwardCallRelationship,
						CALL_TYPE: this.forwardCallType,
						CDFASTRAMDES: this.forwardCDFASTRAMDES,
						CDOPERADOR: forwardCallInstance.value.forwardToForwardCallModal,
						COMMENT: forwardCallInstance.value.observationForwardCallModal,
						DOCUMENT: [],
						NRFASTRAMDES: this.forwardNRFASTRAMDES,
						TXACAOFATRAM: this.forwardTXACAOFATRAM,
						requestType: 'DataSet',
					});
					this.closeModalForwardCall();
					AlertServiceCustom.defaultSaveSuccessAlert();
					this.isLoadingDefault = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.isLoadingDefault = false;
				}
			}
        }
	}

	public resetValidationsForwardCall() {
		const forwardCallInstance = InstanceService.getInstanceZdForm('formForwardCallModal');
		forwardCallInstance.resetValidation();
	}

	public closeModalForwardCall() {
		this.modalForwardCall.hide();
		this.currentCustomerForwardCall = '';
		this.questionLabelForwardCall = '';
		this.disableForwardTo = true;
		this.lastCommentLabel = '';
		this.forwardCallType = '';
		this.forwardCallRelationship = '';
		this.forwardCDFASTRAMDES = '';
		this.forwardNRFASTRAMDES = '';
		this.forwardTXACAOFATRAM = '';
	}
	// END MODAL FORWARD CALL ---

	// START CALL TAB ----
	public setIValidationsEDIT() {
		const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEdit');
		const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
		screenNameInstance.addValidation('required');
		versionIInstance.addValidation('required');
	}

	public removeIValidationsEDIT() {
		const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEdit');
		const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
		screenNameInstance.removeValidation('required');
		versionIInstance.removeValidation('required');
	}

	public setSValidationsEDIT() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
		const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
		productTypeInstance.addValidation('required');
		useLicenseInstance.addValidation('required');
		versionSInstance.addValidation('required');
		releaseInstance.addValidation('required');
		patchInstance.addValidation('required');
	}

	public removeSValidationsEDIT() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
		const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
		productTypeInstance.removeValidation('required');
		useLicenseInstance.removeValidation('required');
		versionSInstance.removeValidation('required');
		releaseInstance.removeValidation('required');
		patchInstance.removeValidation('required');
	}

	public resetValidationsEDIT() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		callFormInstance.resetValidation();
	}

	public onInputClientEditCallModal(params: any) { // CLIENT INPUT EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const productInstance = InstanceService.getInstanceZdSelect('productEdit');
			productInstance.value = '';
			if (params.value) {
				this.resetInputtedValuesEDIT();
				productInstance.value = '';
				productInstance.datasource.filter.CDCLIENTE = params.component.value;
				try {
					await productInstance.datasource.get();
					this.clientNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.clientNOTSelected = true;
					this.productNOTSelected = true;
					this.screenNameNOTSelected = true;
					this.productTypeNOTSelected = true;
					this.versionNOTSelected = true;
					this.releaseNOTSelected = true;
				}
			} else {
				this.clientNOTSelected = true;
				this.productNOTSelected = true;
				this.screenNameNOTSelected = true;
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				this.resetInputtedValuesEDIT();
			}
		}, 350);
	}

	public resetInputtedValuesEDIT() { // I S
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		callFormInstance.value.screenNameEdit = '';
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.productTypeEdit = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEdit = '';
		callFormInstance.value.patchEdit = '';
		callFormInstance.value.useLicenseEdit = '';
	}

	public onInputProductEditCallModal(params: any) { // PRODUCT EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			if (params.value) {
				this.resetInputtedValuesEDIT();
				const customerInstance = InstanceService.getInstanceZdSelect('customerEdit');
				const categoryInstance = InstanceService.getInstanceZdSelect('categoryEdit');
				if (params.component.selectValue.PLATAFORMA === 'MOBILE') {
					const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEdit');
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'I';
					console.log(customerInstance);
					screenNameInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					screenNameInstance.datasource.filter.PRODUCT_ID = params.component.value;
					try {
						await categoryInstance.datasource.get();
						await screenNameInstance.datasource.get();
						this.productNOTSelected = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.productNOTSelected = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
					}
				} else {
					const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
					const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'S';
					productTypeInstance.datasource.filter.PRODUCT_CODE = params.component.value;
					useLicenseInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					try {
						await categoryInstance.datasource.get();
						await productTypeInstance.datasource.get();
						await useLicenseInstance.datasource.get();
						this.productNOTSelected = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.productNOTSelected = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
					}
				}
			} else {
				this.productNOTSelected = true;
				this.screenNameNOTSelected = true;
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				this.resetInputtedValuesEDIT();
			}
		}, 350);
	}

	public onInputScreenNameEditCallModal(params: any) { // SCREEN NAME EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
			if (params.value) {
				versionIInstance.datasource.filter.PRODUCT_ID = params.component.value;
				try {
					await versionIInstance.datasource.get();
					this.screenNameNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.screenNameNOTSelected = true;
				}
			} else {
				this.screenNameNOTSelected = true;
				versionIInstance.value = '';
			}
		}, 350);
	}

	public onInputProductTypeEditCallModal(params: any) { // PRODUCT TYPE EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
			const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
			if (params.value) {
				const productInstance = InstanceService.getInstanceZdSelect('productEdit');
				versionSInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
				versionSInstance.datasource.filter.PRODUCT_CONCAT = `${params.component.selectedData.CDARVPROD}${productInstance.selectedData.SGSISTEMA}`;
				try {
					const version = await versionSInstance.datasource.get();
					if (version.length === 1) {
						this.versionNOTSelected = false;
						versionSInstance.value = version[0].ID;
						releaseInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
						releaseInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
						const release = await releaseInstance.datasource.get();
						if (release.length === 1) {
							this.releaseNOTSelected = false;
							releaseInstance.value = release[0].ID;
							patchInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
							patchInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
							patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
							await patchInstance.datasource.get();
						}
					}
					this.productTypeNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.productTypeNOTSelected = true;
					this.versionNOTSelected = true;
					this.releaseNOTSelected = true;
					versionSInstance.value = '';
					releaseInstance.value = '';
					patchInstance.value = '';
				}
			} else {
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				versionSInstance.value = '';
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		}, 350);
	}

	public onInputVersionSEditCallModal(params: any) { // VERSION S (DELPHI) EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
			const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				releaseInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				releaseInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
				try {
					const release = await releaseInstance.datasource.get();
					if (release.length === 1) {
						releaseInstance.value = release[0].ID;
						this.releaseNOTSelected = false;
						patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
						patchInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
						patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
						await patchInstance.datasource.get();
					}
					this.versionNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.versionNOTSelected = true;
					this.releaseNOTSelected = true;
					releaseInstance.value = '';
					patchInstance.value = '';
				}
			} else {
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		}, 350);
	}

	public onInputReleaseEditCallModal(params: any) { // RELEASE EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
				patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				patchInstance.datasource.filter.PRODUCT_VERSION = versionSInstance.selectedData.ID;
				patchInstance.datasource.filter.PRODUCT_RELEASE = params.component.value;
				try {
					await patchInstance.datasource.get();
					this.releaseNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.releaseNOTSelected = true;
					patchInstance.value = '';
				}
			} else {
				this.releaseNOTSelected = true;
				patchInstance.value = '';
			}
		}, 350);
	}

	public async saveEditCall() { // SAVE EDIT
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		const checkValidation = callFormInstance.validate();
		console.log(checkValidation);
		if (checkValidation) {
			this.isLoadingDefault = true;
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const customerInstance = InstanceService.getInstanceZdSelect('customerEdit');
			if (this.typeIsI) {
				const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEdit');
				try {
					await Http.post('/productCallSave', {
						CALL_RELATIONSHIP: this.callRelationship,
						ID: this.callID,
						REFERENCE: callFormInstance.value.referenceEdit,
						CALL_LABELS: callFormInstance.value.labelsEdit,
						DESCRIPTION: callFormInstance.value.descriptionEdit,
						REPORTED_BY: callFormInstance.value.reportedByEdit,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'MOBILE',
						PROJECT_ID: callFormInstance.value.productEdit,
						SGSISTEMA: null,
						TYPE: 'I',
						PRODUCT_VERSION: '0',
						PRODUCT_RELEASE: '0',
						MENU_ITEM_ID: screenNameInstance.selectedData.ID,
						DEVOPS_ID: screenNameInstance.selectedData.NREXTERNAL_PRODUCT_ID,
						MENU_VERSION_ID: callFormInstance.value.versionEditI,
						CLASSIFICATION: callFormInstance.value.categoryEdit,
						APPOINTMENT_DATE: callFormInstance.value.appointmentDateEdit,
					});
					AlertServiceCustom.defaultSaveSuccessAlert();
					await callGridInstance.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			} else {
				const productInstance = InstanceService.getInstanceZdSelect('productEdit');
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
				try {
					await Http.post('/productCallSave', {
						CALL_RELATIONSHIP: this.callRelationship,
						ID: this.callID,
						REFERENCE: callFormInstance.value.referenceEdit,
						PRODUCT_TYPE_NAME: productTypeInstance.selectedData.LABEL,
						MENU_ITEM: '',
						MENU_VERSION: '',
						NMRAZSOCCLIE: customerInstance.selectedData.NMRAZSOCCLIE,
						CALL_LABELS: callFormInstance.value.labelsEdit,
						DESCRIPTION: callFormInstance.value.descriptionEdit,
						REPORTED_BY: callFormInstance.value.reportedByEdit,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'DELPHI',
						PROJECT_ID: callFormInstance.value.productEdit,
						SGSISTEMA: productInstance.selectedData.SGSISTEMA,
						TYPE: 'S',
						PRODUCT: versionSInstance.selectedData.CDPRODCOMP,
						PRODUCT_VERSION: callFormInstance.value.versionEditS,
						PRODUCT_RELEASE: callFormInstance.value.releaseEdit,
						PRODUCT_TYPE: callFormInstance.value.productTypeEdit,
						CDARVPROD: productTypeInstance.selectedData.CDARVPROD,
						CLASSIFICATION: callFormInstance.value.categoryEdit,
						PRODUCT_PATCH: callFormInstance.value.patchEdit,
						USE_LICENSE: callFormInstance.value.useLicenseEdit,
						APPOINTMENT_DATE: callFormInstance.value.appointmentDateEdit,
					});
					AlertServiceCustom.defaultSaveSuccessAlert();
					await callGridInstance.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			}
			this.isLoadingDefault = false;
		}
	}

	public clickBackCardFormDemandManagement() {
		this.showDemandManagementForm = false;
		// Call Edit
		this.typeIsI = false;
		this.typeIsS = false;
		this.clientNOTSelected = true;
		this.productNOTSelected = true;
		this.screenNameNOTSelected = true;
		this.productTypeNOTSelected = true;
		this.versionNOTSelected = true;
		this.releaseNOTSelected = true;
		this.resetAllValues();
		// Comment
		const tabsInstance = Metadata.getInstance<ITabsBeforeChangeEventParam>('tabMainFormDemandManagement');
		tabsInstance.activeTab = 0;
	}

	public resetAllValues() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		callFormInstance.value.customerIdEdit = '';
		callFormInstance.value.teknisaIdEdit = '';
		callFormInstance.value.reportedByEdit = '';
		callFormInstance.value.referenceEdit = '';
		callFormInstance.value.customerEdit = '';
		callFormInstance.value.productEdit = '';
		callFormInstance.value.categoryEdit = '';
		callFormInstance.value.screenNameEdit = '';
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.productTypeEdit = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEdit = '';
		callFormInstance.value.patchEdit = '';
		callFormInstance.value.useLicenseEdit = '';
		callFormInstance.value.initialAppointmentDateEdit = '';
		callFormInstance.value.appointmentDateEdit = '';
		callFormInstance.value.labelsEdit = '';
		callFormInstance.value.descriptionEdit = '';
	}
	// END CALL TAB ------

	// START COMMENTS TAB --
	public onCreatedIterableComments(params: any) {
		if (params.component.activeTab === 1) {
			setTimeout(async () => {
				await this.updateIterableComments();
			}, 1);
		}
		// DOCUMENTS TAB
		if (params.component.activeTab === 2) {
			setTimeout(async () => {
				const gridAttachments = Metadata.getInstance<any>('gridAttachments');
				console.log(gridAttachments);
				gridAttachments.datasource.filter.CALL_TYPE = this.callType;
				gridAttachments.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
				try {
					await gridAttachments.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			}, 1);
		}
	}

	public async onMountedButtonAddComments() {
		this.modalAddComments = await ModalService.createFromJson('demandManagement/modal/addCommentsDemandManagementModal', true);
	}

	public onDestroyButtonAddComments() {
		this.modalAddComments?.destroy();
	}

	public clickIterableComments(params: any) {
		console.log(params);
	}

	public onMountedBuildFileIterableData(params: any) {
		const id = params.component.name.split('-')[1];
		const iterableCommentsInstance = Metadata.getInstance<IterableComponentRender>('iterableComments');
		const rowData = iterableCommentsInstance.datasource.data.find((row) => row.ID === id);
		if (rowData) {
			if (rowData.ATTACHMENT.length > 0) {
				params.component.datasource.data = rowData.ATTACHMENT;
			}
		}
	}

	public downloadFileComment(params: any) {
		const fileId = params.element.name.split('-')[1];
		const commentId = params.element.name.split('-')[2];
		const fileInstance = Metadata.getInstance<IterableComponentRender>(`iterableFiles-${commentId}`);
		const fileSelected = fileInstance.datasource.data.find((file) => file.ID === fileId);
		if (fileSelected) {
			if (this.callType === 'I') {
				const downloadUri = `${this.baseUrl}/downLoadAttachment?ID=${fileSelected.ID}`;
				window.open(downloadUri);
			} else {
				const downloadUri = `${this.baseUrl}/downLoadAttachmentGEP?FILE_NAME=${fileSelected.FILE_NAME}`;
				window.open(downloadUri);
			}
		}
	}

	public async updateIterableComments() {
		const iterableComments = Metadata.getInstance<IterableComponentRender>('iterableComments');
		iterableComments.datasource.filter.CALL_TYPE = this.callType;
		iterableComments.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
		try {
			await iterableComments.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}
	// END COMMENTS TAB ----

	// START COMMENTS MODAL ----
	public defaultPublicCommentValue = 'N';

	public addCommentButton() {
		this.modalAddComments.show();
		if (this.publicComment) {
			this.defaultPublicCommentValue = 'Y';
		}
	}

	public closeAddCommentsModal() {
		this.modalAddComments.hide();
		this.defaultPublicCommentValue = 'N';
	}

	public get switchButtonDisabledAddCommentsModal() {
		if (this.publicComment) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public async saveAddComment() {
		const addCommentInstance = InstanceService.getInstanceZdForm('formAddCommentsModal');
		const commentValidation = addCommentInstance.validate();
		console.log(addCommentInstance);
        if (commentValidation) {
			this.isLoadingDefault = true;
			const fileInput: any = Metadata.getInstance('fileInputAddCommentsModal');
			if (fileInput.value && fileInput.value.length > 0) {
				let mimeCheck = true;
				const checkFile = fileInput.value.map((file: any) => {
					if ((file.type === 'application/msword'
					|| file.type === 'image/jpeg'
					|| file.type === 'image/png'
					|| file.type === 'application/pdf'
					|| file.type === 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
					|| file.type === 'application/vnd.ms-excel.sheet.macroenabled.12'
					|| file.type === 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
					|| file.type === 'application/vnd.ms-excel'
					|| file.type === 'text/plain'
					|| file.type === 'application/zip')
					&& file.size < 50000000) {
						return FileUpload.buildFileSendObject(file);
					}
					mimeCheck = false;
					return false;
				});
				if (mimeCheck) {
					Promise.all(checkFile).then(async (fileArray) => {
						try {
							await Http.post('/callAttachmentSave', {
								CALL_RELATIONSHIP: this.callRelationship,
								CALL_TYPE: this.callType,
								DESCRIPTION: addCommentInstance.value.commentTextAreaAddCommentsModal,
								PUBLIC: addCommentInstance.value.switchButtonAddCommentsModal,
								DOCUMENT: fileArray,
								requestType: 'DataSet',
							});
							this.updateIterableComments();
							this.closeAddCommentsModal();
							AlertServiceCustom.defaultSaveSuccessAlert();
							this.isLoadingDefault = false;
						} catch (e) {
							DialogServiceCustom.showErrorBackend(e);
							this.isLoadingDefault = false;
						}
					});
				} else {
					AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_TYPE);
					this.isLoadingDefault = false;
				}
			} else {
				try {
					await Http.post('/callAttachmentSave', {
						CALL_RELATIONSHIP: this.callRelationship,
						CALL_TYPE: this.callType,
						DESCRIPTION: addCommentInstance.value.commentTextAreaAddCommentsModal,
						PUBLIC: addCommentInstance.value.switchButtonAddCommentsModal,
						DOCUMENT: [],
						requestType: 'DataSet',
					});
					this.updateIterableComments();
					this.closeAddCommentsModal();
					AlertServiceCustom.defaultSaveSuccessAlert();
					this.isLoadingDefault = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.isLoadingDefault = false;
				}
			}
        }
	}
	// END COMMENTS MODAL ----

	// START DOCUMENTS TAB ---
	public async onMountedGridAttachments() {
		this.modalUploadFile = await ModalService.createFromJson('demandManagement/modal/uploadFileDemandManagementModal', true);
	}

	public onDestroyGridAttachments() {
		this.modalUploadFile.destroy();
	}

	public addClickGridAttachments() {
		this.modalUploadFile.show();
	}

	public downloadFileGrid(params: any) {
		if (this.callType === 'I') {
			const downloadUri = `${this.baseUrl}/downLoadAttachment?ID=${params.row.ID}`;
			window.open(downloadUri);
		} else {
			const downloadUri = `${this.baseUrl}/downLoadAttachmentGEP?FILE_NAME=${params.row.FILE_NAME}`;
			window.open(downloadUri);
		}
	}

	public async deleteAttachmentsGrid(params: any) {
		params.event.preventDefault();
		LoadingService.show('Carregando');
		try {
			await Http.post('deleteProductCallAttachments', params.component.selectedRows);
			params.component.selectedRows = [];
			this.updateFileGrid();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async updateFileGrid() {
		const gridAttachments = Metadata.getInstance<any>('gridAttachments');
		try {
			await gridAttachments.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}
	// END DOCUMENTS TAB -----

	// START DOCUMENTS MODAL ---
	public async saveUploadFileModal() {
		this.isLoadingDefault = true;
		let mimeCheck = true;
        const fileInput: any = Metadata.getInstance('fileInputUploadFileDemandManagementModal');
        if (fileInput.value && fileInput.value.length > 0) {
			const checkFile = fileInput.value.map((file: any) => {
				if ((file.type === 'application/msword'
				|| file.type === 'image/jpeg'
				|| file.type === 'image/png'
				|| file.type === 'application/pdf'
				|| file.type === 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
				|| file.type === 'application/vnd.ms-excel.sheet.macroenabled.12'
				|| file.type === 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
				|| file.type === 'application/vnd.ms-excel'
				|| file.type === 'text/plain'
				|| file.type === 'application/zip')
				&& file.size < 50000000) {
					return FileUpload.buildFileSendObject(file);
				}
				mimeCheck = false;
				return false;
			});
			if (mimeCheck) {
				Promise.all(checkFile).then(async (fileArray) => {
					try {
						await Http.post('/callAttachmentSave', {
							CALL_RELATIONSHIP: this.callRelationship,
							CALL_TYPE: this.callType,
							DOCUMENT: fileArray,
							requestType: 'DataSet',
						});
						this.updateFileGrid();
						this.modalUploadFile.hide();
						AlertServiceCustom.defaultSaveSuccessAlert();
						this.isLoadingDefault = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.isLoadingDefault = false;
					}
				});
			} else {
				AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_TYPE);
				this.isLoadingDefault = false;
			}
        } else {
			AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_EXPECTED);
			this.isLoadingDefault = false;
        }
	}

	public closeUploadFileModal() {
		this.modalUploadFile.hide();
	}
	// END DOCUMENTS MODAL -----
}
